/*
 The MIT License

 Copyright (c) 2009-2010 Paul R. Holser, Jr.

 Permission is hereby granted, free of charge, to any person obtaining
 a copy of this software and associated documentation files (the
 "Software"), to deal in the Software without restriction, including
 without limitation the rights to use, copy, modify, merge, publish,
 distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so, subject to
 the following conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

package com.pholser.util.properties.internal.conversions;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Properties;

import static java.util.Arrays.*;

import com.pholser.util.properties.internal.exceptions.ValueConversionException;

import com.pholser.util.properties.ParsedAs;

class SimpleDateFormatParseValueConverter implements ValueConverter {
    private final ParsedAs parsePatterns;

    public SimpleDateFormatParseValueConverter(ParsedAs parsePatterns) {
        this.parsePatterns = parsePatterns;
    }

    @Override
    public Object convert(String raw, Object... args) {
        for (String each : parsePatterns.value()) {
            try {
                SimpleDateFormat formatter = new SimpleDateFormat(each);
                formatter.setLenient(false);
                return formatter.parse(String.format(raw, args));
            } catch (ParseException ex) {
                // try the next pattern
            }
        }

        throw new ValueConversionException(
            "Could not parse value [" + raw + "] using any of the patterns: " + asList(parsePatterns.value()));
    }

    @Override
    public Object nilValue() {
        return null;
    }

    @Override
    public void resolve(Properties properties) {
        // nothing to do here
    }
}
